/***************************************************************************
 *   Copyright (C) 2018 by Stephane List                                   *
 *   stephane.list@gmail.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "mainwindow.h"
#include "QMessageBox"
#include "ui_mainwindow.h"
#include <QThread>

#include "version.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ui->progressBar_amazon->setValue(0);
    ui->progressBar_apple->setValue(0);
    ui->progressBar_facebook->setValue(0);
    ui->progressBar_google->setValue(0);
    ui->progressBar_stormshield->setValue(0);

    ui->progressBar_amazon->setMaximum(0);
    ui->progressBar_apple->setMaximum(0);
    ui->progressBar_facebook->setMaximum(0);
    ui->progressBar_google->setMaximum(0);
    ui->progressBar_stormshield->setMaximum(0);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::create_cnx(int n)
{
    int prev = ui->progressBar_amazon->maximum();

    ui->progressBar_amazon->setMaximum(prev + n);
    ui->progressBar_apple->setMaximum(prev + n);
    ui->progressBar_facebook->setMaximum(prev + n);
    ui->progressBar_google->setMaximum(prev + n);
    ui->progressBar_stormshield->setMaximum(prev + n);

    for (int i=0; i<n ; i++)
    {
        QNetworkAccessManager *manager = new QNetworkAccessManager();
        connect(manager, SIGNAL(finished(QNetworkReply*)), this,
                SLOT(replyFinished(QNetworkReply*)));

        create_cnx_host(manager, "https://www.google.com");
        create_cnx_host(manager, "https://www.apple.com");
        create_cnx_host(manager, "https://www.facebook.com");
        create_cnx_host(manager, "https://www.amazon.com");
        create_cnx_host(manager, "https://www.stormshield.com");
    }
}

void MainWindow::create_cnx_host(QNetworkAccessManager *manager, const QString h)
{
    QNetworkRequest request;
    QSslConfiguration config = QSslConfiguration::defaultConfiguration();

    config.setProtocol(QSsl::TlsV1_2);
    request.setSslConfiguration(config);
    request.setUrl(QUrl(h));
    request.setHeader(QNetworkRequest::ServerHeader, "application/json");
    manager->get(request);
    QThread::msleep(20);
}

void MainWindow::replyFinished(QNetworkReply *reply)
{
    QString h;
    h = reply->url().host();

    if (h.contains("google")) {
        int val = ui->progressBar_google->value();
        ui->progressBar_google->setValue(val + 1);
    } else if (h.contains("apple")) {
        int val = ui->progressBar_apple->value();
        ui->progressBar_apple->setValue(val + 1);
    } else if (h.contains("facebook")) {
        int val = ui->progressBar_facebook->value();
        ui->progressBar_facebook->setValue(val + 1);
    } else if (h.contains("amazon")) {
        int val = ui->progressBar_amazon->value();
        ui->progressBar_amazon->setValue(val + 1);
    } else if (h.contains("stormshield")) {
        int val = ui->progressBar_stormshield->value();
        ui->progressBar_stormshield->setValue(val + 1);
    }
    ui->plainTextEdit->appendPlainText(h);
    reply->readAll();
    reply->deleteLater();

    ui->lcdNumber->display(ui->lcdNumber->value()+1);
}

void MainWindow::on_action_About_QT_triggered()
{
    QMessageBox::aboutQt(this, "gafas");
}

void MainWindow::on_actionAbout_triggered()
{
    QMessageBox::about(this,
                       tr("About"),
                       tr("<p><b>gafas</b></p>"
                          "<p>Version gafas: %1"
                          "<p>Version %2: %3.%4"
                          "</p>")
                       .arg(VERSION)
                   #ifdef __GNUC__
                       .arg("GCC")
                       .arg(__GNUC__)
                       .arg(__GNUC_MINOR__)
                   #elif defined(_MSC_VER)
                       .arg("VC++")
                       .arg(_MSC_VER / 100)
                       .arg(_MSC_VER % 100,2,10,QChar('0'))
                   #else
                       .arg("Compiler")
                       .arg("?")
                       .arg("?")
                   #endif
                       );

}

void MainWindow::on_action_Quit_triggered()
{
    QApplication::quit();
}

void MainWindow::on_action_URLs_triggered()
{

}

void MainWindow::on_pushButton_1_clicked()
{
    create_cnx(100);
}

void MainWindow::on_pushButton_2_clicked()
{
    create_cnx(20);
}

void MainWindow::on_pushButton_3_clicked()
{
    create_cnx(30);
}

void MainWindow::on_pushButton_4_clicked()
{
    create_cnx(40);
}
